package atom

import (
	"encoding/xml"
	"fmt"
)

type CommonAttributes struct {
	Base                string      `xml:"base,attr,omitempty"` // IRI
	Lang                string      `xml:"lang,attr,omitempty"` // LanguageTag
	UndefinedAttributes []*xml.Attr `xml:",attr,omitempty"`
}

// NewCommonAttributes creates a new set of CommonAttributes. It returns a
// *CommonAttributes.
func newCommonAttributes() *CommonAttributes {
	return new(CommonAttributes)
}

// AddAttribute adds the attribute to the CommonAttributes.
func (c *CommonAttributes) AddAttribute(name, value string) {
	if c.UndefinedAttributes == nil {
		c.UndefinedAttributes = make([]*xml.Attr, 1)
		c.UndefinedAttributes[0] = &xml.Attr{Name: xml.Name{Local: name}, Value: value}
	} else {
		c.UndefinedAttributes = append(c.UndefinedAttributes, &xml.Attr{Name: xml.Name{Local: name}, Value: value})
	}
}

// DeleteAttribute deletes the attribute from the CommonAttributes. It return an
// error.
func (c *CommonAttributes) DeleteAttribute(id int) error {
	length := len(c.UndefinedAttributes)
	if id > length {
		return fmt.Errorf("error deleting undefined attribute from common attributes %v: id %v out of range %v", c, id, length)
	}

	c.UndefinedAttributes = append(c.UndefinedAttributes[:id], c.UndefinedAttributes[id+1:]...)
	return nil
}

// Check checks the CommonAttributes for incompatibilities with RFC4287. It
// returns an error.
func (c *CommonAttributes) Check() error {
	for i, u := range c.UndefinedAttributes {
		if u.Name.Local == "" {
			return fmt.Errorf("xml name of undefined attribute %v empty", i)
		}
		if u.Value == "" {
			return fmt.Errorf("value of undefined attribute %v empty", i)
		}
	}

	return nil
}
